@@ -27,7 +27,6 @@ of the frame of the GUI for the Bumps application.
 
 # ==============================================================================
 
-from __future__ import division
 import os
 import threading
 from copy import deepcopy
@@ -36,25 +35,24 @@ import wx
 import wx.aui
 
 from .. import plugin
-from ..cli import load_model, load_best
-from ..util import redirect_console
+from ..cli import load_best, load_model
 from ..dream import stats as dream_stats
-
+from ..util import redirect_console
+from . import signal
+from .convergence_view import ConvergenceView
+from .fit_dialog import show_fit_config
+from .fit_thread import EVT_FIT_COMPLETE, EVT_FIT_PROGRESS, FitThread
+from .log_view import LogView
+from .parameter_view import ParameterView
 from .plot_view import PlotView
 from .summary_view import SummaryView
-from .parameter_view import ParameterView
-from .log_view import LogView
-from .convergence_view import ConvergenceView
-from .uncertainty_view import CorrelationView, UncertaintyView, TraceView, ModelErrorView
-from .fit_dialog import show_fit_config
-from .fit_thread import (FitThread, EVT_FIT_PROGRESS, EVT_FIT_COMPLETE)
+from .uncertainty_view import CorrelationView, ModelErrorView, TraceView, UncertaintyView
 from .util import nice
-from . import signal
 from .utilities import get_bitmap, phoenix
 
 # File selection strings.
 MODEL_EXT = ".pickle"
-MODEL_FILES = "Model files (*%s)|*%s"%(MODEL_EXT,MODEL_EXT)
+MODEL_FILES = "Model files (*%s)|*%s" % (MODEL_EXT, MODEL_EXT)
 PYTHON_FILES = "Script files (*.py)|*.py"
 DATA_FILES = "Data files (*.dat)|*.dat"
 TEXT_FILES = "Text files (*.txt)|*.txt"
@@ -64,7 +62,8 @@ PARS_FILES = "Parameter files (*.par)|*.par"
 # Custom colors.
 WINDOW_BKGD_COLOUR = "#ECE9D8"
 
-#==============================================================================
+# ==============================================================================
+
 
 class AppPanel(wx.Panel):
     """
@@ -94,8 +93,8 @@ class AppPanel(wx.Panel):
 
         # Add data menu
         mb = frame.GetMenuBar()
-        data_view = self.view['data']
-        if hasattr(data_view, 'menu'):
+        data_view = self.view["data"]
+        if hasattr(data_view, "menu"):
             mb.Append(data_view.menu(), data_view.title)
 
         # Create a PubSub receiver.
@@ -110,7 +109,7 @@ class AppPanel(wx.Panel):
         # _fit_abort and maybe _uncertainty_state are managed by fit_lock
         self.fit_lock = threading.Lock()
         self._fit_abort = False
-        #self._uncertainty_state = None
+        # self._uncertainty_state = None
         self.fit_thread = None
         self.fit_config = None
 
@@ -123,82 +122,61 @@ class AppPanel(wx.Panel):
 
         file_menu_id = mb.FindMenu("File")
         file_menu = mb.GetMenu(file_menu_id)
-        #help_menu = mb.GetMenu(mb.FindMenu("Help"))
+        # help_menu = mb.GetMenu(mb.FindMenu("Help"))
 
         # Add items to the "File" menu (prepending them in reverse order).
         # Grey out items that are not currently implemented.
         file_menu.PrependSeparator()
 
-        _item = file_menu.Prepend(wx.ID_ANY,
-                                  "E&xport Results ...",
-                                  "Save theory, data and parameters")
+        _item = file_menu.Prepend(wx.ID_ANY, "E&xport Results ...", "Save theory, data and parameters")
         frame.Bind(wx.EVT_MENU, self.OnFileExportResults, _item)
 
-        _item = file_menu.Prepend(wx.ID_ANY,
-                                  "&Reload",
-                                  "Reload the existing model")
+        _item = file_menu.Prepend(wx.ID_ANY, "&Reload", "Reload the existing model")
         frame.Bind(wx.EVT_MENU, self.OnFileReload, _item)
 
-        _item = file_menu.Prepend(wx.ID_SAVEAS,
-                                  "Save &As",
-                                  "Save model as another name")
+        _item = file_menu.Prepend(wx.ID_SAVEAS, "Save &As", "Save model as another name")
         frame.Bind(wx.EVT_MENU, self.OnFileSaveAs, _item)
-        #file_menu.Enable(id=wx.ID_SAVEAS, enable=False)
-        _item = file_menu.Prepend(wx.ID_SAVE,
-                                  "&Save",
-                                  "Save model")
+        # file_menu.Enable(id=wx.ID_SAVEAS, enable=False)
+        _item = file_menu.Prepend(wx.ID_SAVE, "&Save", "Save model")
         frame.Bind(wx.EVT_MENU, self.OnFileSave, _item)
-        #file_menu.Enable(id=wx.ID_SAVE, enable=False)
-        _item = file_menu.Prepend(wx.ID_OPEN,
-                                  "&Open",
-                                  "Open existing model")
+        # file_menu.Enable(id=wx.ID_SAVE, enable=False)
+        _item = file_menu.Prepend(wx.ID_OPEN, "&Open", "Open existing model")
         frame.Bind(wx.EVT_MENU, self.OnFileOpen, _item)
-        #file_menu.Enable(id=wx.ID_OPEN, enable=False)
-        _item = file_menu.Prepend(wx.ID_ANY,
-                                  "Apply &Pararameters",
-                                  "Apply parameters from .par file to loaded model")
+        # file_menu.Enable(id=wx.ID_OPEN, enable=False)
+        _item = file_menu.Prepend(wx.ID_ANY, "Apply &Pararameters", "Apply parameters from .par file to loaded model")
         frame.Bind(wx.EVT_MENU, self.OnParsFileOpen, _item)
-        _item = file_menu.Prepend(wx.ID_NEW,
-                                  "&New",
-                                  "Create new model")
+        _item = file_menu.Prepend(wx.ID_NEW, "&New", "Create new model")
         frame.Bind(wx.EVT_MENU, self.OnFileNew, _item)
-        #file_menu.Enable(id=wx.ID_NEW, enable=False)
+        # file_menu.Enable(id=wx.ID_NEW, enable=False)
 
         # Add 'Fitting' menu to the menu bar and define its options.
         # Items are initially greyed out, but will be enabled after a script
         # is loaded.
         fit_menu = self.fit_menu = wx.Menu()
 
-        _item = fit_menu.Append(wx.ID_ANY,
-                                "Start",
-                                "Start fitting operation")
+        _item = fit_menu.Append(wx.ID_ANY, "Start", "Start fitting operation")
         frame.Bind(wx.EVT_MENU, self.OnFitStart, _item)
         fit_menu.Enable(id=_item.GetId(), enable=False)
         self.fit_menu_start = _item
 
-        _item = fit_menu.Append(wx.ID_ANY,
-                                "Stop",
-                                "Stop fitting operation")
+        _item = fit_menu.Append(wx.ID_ANY, "Stop", "Stop fitting operation")
         frame.Bind(wx.EVT_MENU, self.OnFitStop, _item)
         fit_menu.Enable(id=_item.GetId(), enable=False)
         self.fit_menu_stop = _item
 
-        _item = fit_menu.Append(wx.ID_ANY,
-                                "&Options ...",
-                                "Edit fitting options")
+        _item = fit_menu.Append(wx.ID_ANY, "&Options ...", "Edit fitting options")
         frame.Bind(wx.EVT_MENU, self.OnFitOptions, _item)
         self.fit_menu_options = _item
 
-        #_item = fit_menu.Append(wx.ID_ANY,
+        # _item = fit_menu.Append(wx.ID_ANY,
         #                        "&Save ...",
         #                        "Save fit results")
-        #frame.Bind(wx.EVT_MENU, self.OnFitSave, _item)
-        #fit_menu.Enable(id=_item.GetId(), enable=False)
+        # frame.Bind(wx.EVT_MENU, self.OnFitSave, _item)
+        # fit_menu.Enable(id=_item.GetId(), enable=False)
         self.fit_menu_options = _item
 
         mb.Append(fit_menu, "&Fitting")
 
-
     def init_toolbar(self, frame):
         """Populates the tool bar."""
         self.tb = frame.GetToolBar()
@@ -208,28 +186,20 @@ class AppPanel(wx.Panel):
         start_bmp = get_bitmap("start_fit.png", wx.BITMAP_TYPE_PNG)
         stop_bmp = get_bitmap("stop_fit.png", wx.BITMAP_TYPE_PNG)
 
-        _tool = self._add_tool(script_bmp,
-                               "Open model",
-                               "Load model from script")
+        _tool = self._add_tool(script_bmp, "Open model", "Load model from script")
         frame.Bind(wx.EVT_TOOL, self.OnFileOpen, _tool)
-        _tool = self._add_tool(reload_bmp,
-                               "Reload model",
-                               "Reload model from script")
+        _tool = self._add_tool(reload_bmp, "Reload model", "Reload model from script")
         frame.Bind(wx.EVT_TOOL, self.OnFileReload, _tool)
         # TODO: add reload
 
         self.tb.AddSeparator()
 
-        _tool = self._add_tool(start_bmp,
-                               "Start Fit",
-                               "Start fitting operation")
+        _tool = self._add_tool(start_bmp, "Start Fit", "Start fitting operation")
         frame.Bind(wx.EVT_TOOL, self.OnFitStart, _tool)
         self.tb.EnableTool(_tool.GetId(), False)
         self.tb_start = _tool
 
-        _tool = self._add_tool(stop_bmp,
-                               "Stop Fit",
-                               "Stop fitting operation")
+        _tool = self._add_tool(stop_bmp, "Stop Fit", "Stop fitting operation")
         frame.Bind(wx.EVT_TOOL, self.OnFitStop, _tool)
         self.tb.EnableTool(_tool.GetId(), False)
         self.tb_stop = _tool
@@ -255,26 +225,34 @@ class AppPanel(wx.Panel):
         self.aui = wx.aui.AuiNotebook(self)
         self.aui.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.OnViewTabClose)
         self.view_constructor = {
-            'data': plugin.data_view(),
-            'model': plugin.model_view(),
-            'parameter': ParameterView,
-            'summary': SummaryView,
-            'log': LogView,
-            'convergence': ConvergenceView,
-            'uncertainty': UncertaintyView,
-            'correlation': CorrelationView,
-            'trace': TraceView,
-            'error': ModelErrorView,
-            }
-        self.view_list = ['data','model','parameter',
-                          'summary','log','convergence',
-                          'uncertainty','correlation','trace','error']
+            "data": plugin.data_view(),
+            "model": plugin.model_view(),
+            "parameter": ParameterView,
+            "summary": SummaryView,
+            "log": LogView,
+            "convergence": ConvergenceView,
+            "uncertainty": UncertaintyView,
+            "correlation": CorrelationView,
+            "trace": TraceView,
+            "error": ModelErrorView,
+        }
+        self.view_list = [
+            "data",
+            "model",
+            "parameter",
+            "summary",
+            "log",
+            "convergence",
+            "uncertainty",
+            "correlation",
+            "trace",
+            "error",
+        ]
         self.view = {}
         for v in self.view_list:
             if self.view_constructor[v]:
-                self.view[v] = self.view_constructor[v](self.aui,
-                                                        size=(600,600))
-                self.aui.AddPage(self.view[v],self.view_constructor[v].title)
+                self.view[v] = self.view_constructor[v](self.aui, size=(600, 600))
+                self.aui.AddPage(self.view[v], self.view_constructor[v].title)
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer.Add(self.aui, 1, wx.EXPAND)
         self.SetSizer(sizer)
@@ -288,18 +266,17 @@ class AppPanel(wx.Panel):
 
     def OnViewTabClose(self, evt):
         win = self.aui.GetPage(evt.GetSelection())
-        #print "Closing tab",win.GetId()
+        # print "Closing tab",win.GetId()
         for k, w in self.view.items():
             if w == win:
                 tag = k
                 break
         else:
             raise RuntimeError("Lost track of view")
-        #print "creating external frame"
+        # print "creating external frame"
         state = self.view[tag].get_state()
         constructor = self.view_constructor[tag]
-        frame = wx.Frame(self, title=constructor.title,
-                         size=constructor.default_size)
+        frame = wx.Frame(self, title=constructor.title, size=constructor.default_size)
         panel = constructor(frame)
         self.view[tag] = panel
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -310,10 +287,9 @@ class AppPanel(wx.Panel):
         panel.set_state(state)
         evt.Skip()
 
-
     def OnViewFrameClose(self, evt):
         win = evt.GetEventObject()
-        #print "Closing frame",win.GetId()
+        # print "Closing frame",win.GetId()
         for k, w in self.view.items():
             if w.GetParent() == win:
                 tag = k
@@ -331,7 +307,7 @@ class AppPanel(wx.Panel):
     # model viewer interface
     def OnLogMessage(self, message):
         for v in self.view.values():
-            if hasattr(v, 'log_message'):
+            if hasattr(v, "log_message"):
                 v.log_message(message)
 
     def OnModelNew(self, model):
@@ -339,12 +315,12 @@ class AppPanel(wx.Panel):
 
     def OnModelChange(self, model):
         for v in self.view.values():
-            if hasattr(v, 'update_model'):
+            if hasattr(v, "update_model"):
                 v.update_model(model)
 
     def OnModelSetpar(self, model):
         for v in self.view.values():
-            if hasattr(v, 'update_parameters'):
+            if hasattr(v, "update_parameters"):
                 v.update_parameters(model)
 
     def OnFileNew(self, event):
@@ -352,12 +328,14 @@ class AppPanel(wx.Panel):
 
     def OnFileOpen(self, event):
         # Load the script which will contain model definition and data.
-        dlg = wx.FileDialog(self,
-                            message="Select File",
-                            #defaultDir=os.getcwd(),
-                            #defaultFile="",
-                            wildcard=(ALL_FILES),
-                            style=wx.FD_OPEN|wx.FD_CHANGE_DIR)
+        dlg = wx.FileDialog(
+            self,
+            message="Select File",
+            # defaultDir=os.getcwd(),
+            # defaultFile="",
+            wildcard=(ALL_FILES),
+            style=wx.FD_OPEN | wx.FD_CHANGE_DIR,
+        )
 
         # Wait for user to close the dialog.
         status = dlg.ShowModal()
@@ -367,15 +345,17 @@ class AppPanel(wx.Panel):
         # Process file if user clicked okay.
         if status == wx.ID_OK:
             self.load_model(path)
-    
+
     def OnParsFileOpen(self, event):
         # Load a parameters (.pars) file to apply to current problem.
-        dlg = wx.FileDialog(self,
-                            message="Select Parameters File",
-                            #defaultDir=os.getcwd(),
-                            #defaultFile="",
-                            wildcard=PARS_FILES,
-                            style=wx.FD_OPEN)
+        dlg = wx.FileDialog(
+            self,
+            message="Select Parameters File",
+            # defaultDir=os.getcwd(),
+            # defaultFile="",
+            wildcard=PARS_FILES,
+            style=wx.FD_OPEN,
+        )
 
         # Wait for user to close the dialog.
         status = dlg.ShowModal()
@@ -386,26 +366,27 @@ class AppPanel(wx.Panel):
         if status == wx.ID_OK:
             self.apply_parameters(path)
 
-
     def OnFileReload(self, event):
-        path = getattr(self, '_reload_path', self.model.path)
+        path = getattr(self, "_reload_path", self.model.path)
         self.load_model(path)
 
     def OnFileSave(self, event):
         if self.model is not None:
             # Force file extension to be MODEL_EXT
-            self.model.path = os.path.splitext(self.model.path)[0]+MODEL_EXT
+            self.model.path = os.path.splitext(self.model.path)[0] + MODEL_EXT
             self.save_model(self.model.path)
         else:
             self.OnFileSaveAs(event)
 
     def OnFileSaveAs(self, event):
-        dlg = wx.FileDialog(self,
-                            message="Select File",
-                            defaultDir=os.getcwd(),
-                            defaultFile="",
-                            wildcard=(MODEL_FILES+"|"+ALL_FILES),
-                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR|wx.FD_OVERWRITE_PROMPT)
+        dlg = wx.FileDialog(
+            self,
+            message="Select File",
+            defaultDir=os.getcwd(),
+            defaultFile="",
+            wildcard=(MODEL_FILES + "|" + ALL_FILES),
+            style=wx.FD_SAVE | wx.FD_CHANGE_DIR | wx.FD_OVERWRITE_PROMPT,
+        )
         # Wait for user to close the dialog.
         status = dlg.ShowModal()
         path = dlg.GetPath()
@@ -413,15 +394,11 @@ class AppPanel(wx.Panel):
 
         # Process file if user clicked okay.
         if status == wx.ID_OK:
-            self.model.path = os.path.splitext(path)[0]+MODEL_EXT
+            self.model.path = os.path.splitext(path)[0] + MODEL_EXT
             self.save_model(self.model.path)
 
-
     def OnFileExportResults(self, event):
-        dlg = wx.DirDialog(self,
-                           message="Export results",
-                           defaultPath=os.getcwd(),
-                           style=wx.DD_DEFAULT_STYLE)
+        dlg = wx.DirDialog(self, message="Export results", defaultPath=os.getcwd(), style=wx.DD_DEFAULT_STYLE)
         # Wait for user to close the dialog.
         status = dlg.ShowModal()
         path = dlg.GetPath()
@@ -463,7 +440,7 @@ class AppPanel(wx.Panel):
             # Number of seconds between updates to the GUI, or 0 for no updates
             convergence_update=5,
             uncertainty_update=3600,
-            )
+        )
         self.fit_thread.start()
         self.sb.SetStatusText("Fit status: Running", 3)
 
@@ -504,26 +481,28 @@ class AppPanel(wx.Panel):
 
     def OnFitComplete(self, event):
         if self.fit_thread is not None:
-            self.fit_thread.join(1) # 1 second timeout on join
+            self.fit_thread.join(1)  # 1 second timeout on join
             if self.fit_thread.is_alive():
                 signal.log_message(message="fit thread failed to complete")
         self.fit_thread = None
-        chisq = nice(2*event.value/event.problem.dof)
+        chisq = event.problem.chisq_str(nllf=event.value)
         event.problem.setp(event.point)
         signal.update_parameters(model=event.problem)
-        signal.log_message(message="done with chisq %g"%chisq)
+        signal.log_message(message="done with chisq %s" % chisq)
         signal.log_message(message=event.info)
         self.sb.SetStatusText("Fit status: Complete", 3)
         beep()
 
     def OnFitSave(self, event):
         raise NotImplementedError()
-        dlg = wx.FileDialog(self,
-                            message="Fit results",
-                            defaultDir=os.getcwd(),
-                            defaultFile="",
-                            wildcard=(MODEL_FILES+"|"+ALL_FILES),
-                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR|wx.FD_OVERWRITE_PROMPT)
+        dlg = wx.FileDialog(
+            self,
+            message="Fit results",
+            defaultDir=os.getcwd(),
+            defaultFile="",
+            wildcard=(MODEL_FILES + "|" + ALL_FILES),
+            style=wx.FD_SAVE | wx.FD_CHANGE_DIR | wx.FD_OVERWRITE_PROMPT,
+        )
         # Wait for user to close the dialog.
         status = dlg.ShowModal()
         path = dlg.GetPath()
@@ -533,17 +512,20 @@ class AppPanel(wx.Panel):
         if status == wx.ID_OK:
             self.save_fit(path)
 
-
     def OnFitProgress(self, event):
         self._fit_progress(event)
         return
 
         # Note: The following code was used to debugging plotting memory
         # and speed issues. It is here in case it might be useful again.
-        if event.message == 'uncertainty_final':
-            event.message = 'uncertainty_update'  # don't do model uncertainty
+        if event.message == "uncertainty_final":
+            event.message = "uncertainty_update"  # don't do model uncertainty
             n = 0
-            import psutil, time, gc
+            import gc
+            import time
+
+            import psutil
+
             pid = os.getpid()
             proc = psutil.Process()
             t0 = time.perf_counter()
@@ -551,39 +533,38 @@ class AppPanel(wx.Panel):
             while True:
                 self._fit_progress(event)
                 t1 = time.perf_counter()
-                msg = ("========== cycle %d pid %d time %.3f s ==========="
-                       % (n, pid, t1 - t0))
+                msg = "========== cycle %d pid %d time %.3f s ===========" % (n, pid, t1 - t0)
                 t0 = t1
-                #signal.log_message(msg)
-                #signal.log_message(f"memory {proc.memory_full_info()}")
+                # signal.log_message(msg)
+                # signal.log_message(f"memory {proc.memory_full_info()}")
                 gc.collect()
                 print(msg)
                 print("memory", proc.memory_full_info())
                 n += 1
                 wx.Yield()
-                #if n > 100: break
+                # if n > 100: break
 
     def _fit_progress(self, event):
-        if event.message == 'progress':
-            chisq = nice(2*event.value/event.problem.dof)
-            message = "step %5d chisq %g"%(event.step, chisq)
+        if event.message == "progress":
+            chisq = event.problem.chisq_str(nllf=event.value)
+            message = "step %5d chisq %s" % (event.step, chisq)
             signal.log_message(message=message)
-        elif event.message == 'improvement':
+        elif event.message == "improvement":
             event.problem.setp(event.point)
             event.problem.model_update()
             signal.update_parameters(model=event.problem)
-        elif event.message == 'convergence_update':
-            self.view['convergence'].OnFitProgress(event)
-        elif event.message in ('uncertainty_update', 'uncertainty_final'):
+        elif event.message == "convergence_update":
+            self.view["convergence"].OnFitProgress(event)
+        elif event.message in ("uncertainty_update", "uncertainty_final"):
             # TODO: revert to event.uncertainty_state if memory leak isn't fixed
             # Note: uncertainty_state is updated directly by the fit thread.
             # It's a copy protected by fit_lock so it should be self-consistent.
             state = self.uncertainty_state
             stats = dream_stats.var_stats(state.draw())
-            self.console['state'] = self.uncertainty_state
-            self.view['uncertainty'].fit_progress(event.problem, state, stats)
-            self.view['correlation'].fit_progress(event.problem, state)
-            self.view['trace'].fit_progress(event.problem, state)
+            self.console["state"] = self.uncertainty_state
+            self.view["uncertainty"].fit_progress(event.problem, state, stats)
+            self.view["correlation"].fit_progress(event.problem, state)
+            self.view["trace"].fit_progress(event.problem, state)
             # TODO: send parameter stats to a view
             # It would be nice to have a "log scale" log of parameter history,
             # where the density of entries decreases exponentially over time.
@@ -592,46 +573,50 @@ class AppPanel(wx.Panel):
             # forward and backward and see changes over time. Use box plots
             # to represent mode, median, mean, 68% and 95% intervals. Export
             # latest or all to CSV or text for pasting into word or excel.
-            if event.message == 'uncertainty_final':
-                self.view['error'].fit_progress(event.problem, state)
+            if event.message == "uncertainty_final":
+                self.view["error"].fit_progress(event.problem, state)
                 # Format the final variable nicely and show them on the console.
                 # Note: only done at the end because GUI gets overwhelmed when
                 # the text control has too much text.
                 signal.log_message(dream_stats.format_vars(stats))
         else:
-            raise ValueError("Unknown fit progress message "+event.message)
+            raise ValueError("Unknown fit progress message " + event.message)
 
     def new_model(self):
         from ..plugin import new_model as gen
+
         self.set_model(gen())
 
     def load_model(self, path):
         self._reload_path = path
         model = load_model(path)
         signal.model_new(model=model)
-    
+
     def apply_parameters(self, path):
         load_best(self.model, path)
         # TODO: remove this once LHS randomization is controlled from the command line. Ticket #51
-        if hasattr(self.model, 'undefined'):
+        if hasattr(self.model, "undefined"):
             del self.model.undefined
         signal.update_parameters(model=self.model)
 
     def save_model(self, path):
         try:
             import dill
+
             dump = lambda obj, fid: dill.dump(obj, fid, recurse=True)
         except ImportError:
             from pickle import dump
         try:
-            if hasattr(self.model, 'save_json'):
+            if hasattr(plugin, "save_json"):
+                plugin.save_json(self.model, path)
+            elif hasattr(self.model, "save_json"):
                 self.model.save_json(path)
-            with open(path,'wb') as fid:
+            with open(path, "wb") as fid:
                 dump(self.model, fid)
         except Exception:
             import traceback
-            signal.log_message(message=traceback.format_exc())
 
+            signal.log_message(message=traceback.format_exc())
 
     def save_results(self, path):
         output_path = os.path.join(path, self.model.name)
@@ -644,25 +629,23 @@ class AppPanel(wx.Panel):
         self.model.save(output_path)
 
         # Save a snapshot of the model that can (hopefully) be reloaded
-        self.save_model(output_path+MODEL_EXT)
+        self.save_model(output_path + MODEL_EXT)
 
         # Save the current state of the parameters
-        with redirect_console(output_path+".out"):
+        with redirect_console(output_path + ".out"):
             self.model.show()
-        pardata = "".join("%s %.15g\n"%(name, value) for name, value in
-                          zip(self.model.labels(), self.model.getp()))
-        open(output_path+".par",'wt').write(pardata)
+        pardata = "".join("%s %.15g\n" % (name, value) for name, value in zip(self.model.labels(), self.model.getp()))
+        open(output_path + ".par", "wt").write(pardata)
 
         # Produce model plots
         self.model.plot(figfile=output_path)
 
         # Produce uncertainty plots
         if self.uncertainty_state is not None:
-            with redirect_console(output_path+".err"):
+            with redirect_console(output_path + ".err"):
                 self.uncertainty_state.show(figfile=output_path)
             self.uncertainty_state.save(output_path)
 
-
     def _add_measurement_type(self, type):
         """
         Add the panels needed to view a measurement of the given type.
@@ -671,11 +654,11 @@ class AppPanel(wx.Panel):
         """
         name = type.__name__
         if type not in self.data_tabs:
-            tab = self.data_notebook.add_tab(type, name+" Data")
-            constructor = getattr(type, 'data_panel', PlotView)
+            tab = self.data_notebook.add_tab(type, name + " Data")
+            constructor = getattr(type, "data_panel", PlotView)
             constructor(tab)
-        if type not in self.model_notebook and hasattr(type, 'model_panel'):
-            tab = self.model_notebook.add_tab(type, name+" Model")
+        if type not in self.model_notebook and hasattr(type, "model_panel"):
+            tab = self.model_notebook.add_tab(type, name + " Model")
             type.model_panel(tab)
 
     def _view_problem(self, problem):
@@ -683,14 +666,15 @@ class AppPanel(wx.Panel):
         Set the model and data views to those necessary to display the problem.
         """
         # What types of measurements do we have?
-        models = problem.models if hasattr(problem,'models') else [problem]
+        models = problem.models if hasattr(problem, "models") else [problem]
         types = set(p.fitness.__class__ for p in models)
-        for p in types: self._add_measurement_type(p)
+        for p in types:
+            self._add_measurement_type(p)
 
         # Show only the relevant views
-        for p,tab in self.data_notebook.tabs():
+        for p, tab in self.data_notebook.tabs():
             tab.Show(p in types)
-        for p,tab in self.model_notebook.tabs():
+        for p, tab in self.model_notebook.tabs():
             tab.Show(p in types)
 
     def set_model(self, model):
@@ -699,9 +683,9 @@ class AppPanel(wx.Panel):
 
         # Point all of our views at the new model
         for v in self.view.values():
-            if hasattr(v,'set_model'):
+            if hasattr(v, "set_model"):
                 v.set_model(model)
-        self.console['model'] = model
+        self.console["model"] = model
 
         # Enable appropriate menu items.
         self.fit_menu.Enable(id=self.fit_menu_start.GetId(), enable=True)
@@ -711,22 +695,27 @@ class AppPanel(wx.Panel):
         # Enable appropriate toolbar items.
         self.tb.EnableTool(self.tb_start.GetId(), True)
         self.tb.EnableTool(self.tb_stop.GetId(), True)
-        if hasattr(model, 'path'):
-            signal.log_message(message="loaded "+model.path)
-            self.GetTopLevelParent().SetTitle("Bumps: %s"%model.name)
+        if hasattr(model, "broken_constraints"):
+            signal.log_message(message="Unsatisfied constraints: \n\t%s" % (",\n\t".join(model.broken_constraints)))
+        if hasattr(model, "path"):
+            signal.log_message(message="loaded " + model.path)
+            self.GetTopLevelParent().SetTitle("Bumps: %s" % model.name)
         else:
             signal.log_message(message="new model")
             self.GetTopLevelParent().SetTitle("Bumps")
 
+
 SOUND = None
+
+
 def beep():
     """
     Play fit completion sound.
     """
     wx.Bell()
     ## FIXME why doesn't sound work?
-    #global SOUND
-    #if SOUND is None:
+    # global SOUND
+    # if SOUND is None:
     #    SOUND = wx.Sound(resource('done.wav'))
-    #if SOUND.IsOk():
+    # if SOUND.IsOk():
     #    SOUND.Play(wx.SOUND_ASYNC)
